@@ -39,7 +39,7 @@
  *
  */
 
-/* $Id: global.c,v 1.17 2004/11/23 17:40:15 shadow Exp $ */
+/* $Id: global.c,v 1.2.2.15 2005/12/13 19:35:50 murch Exp $ */
 
 #include <config.h>
 
@@ -82,16 +82,16 @@ static enum {
 static int cyrus_init_nodb = 0;
 
 int config_implicitrights;        /* "lca" */
+unsigned long config_metapartition_files;    /* 0 */
 struct cyrusdb_backend *config_mboxlist_db;
 struct cyrusdb_backend *config_quota_db;
 struct cyrusdb_backend *config_subscription_db;
 struct cyrusdb_backend *config_annotation_db;
 struct cyrusdb_backend *config_seenstate_db;
+struct cyrusdb_backend *config_mboxkey_db;
 struct cyrusdb_backend *config_duplicate_db;
 struct cyrusdb_backend *config_tlscache_db;
-#ifdef WITH_PTS
 struct cyrusdb_backend *config_ptscache_db;
-#endif
 
 /* Called before a cyrus application starts (but after command line parameters
  * are read) */
@@ -163,6 +163,8 @@ int cyrus_init(const char *alt_config, const char *ident, unsigned flags)
     config_implicitrights =
 	cyrus_acl_strtomask(config_getstring(IMAPOPT_IMPLICIT_OWNER_RIGHTS));
 
+    config_metapartition_files = config_getbitfield(IMAPOPT_METAPARTITION_FILES);
+
     if (!cyrus_init_nodb) {
 	/* lookup the database backends */
 	config_mboxlist_db =
@@ -175,14 +177,14 @@ int cyrus_init(const char *alt_config, const char *ident, unsigned flags)
 	    cyrusdb_fromname(config_getstring(IMAPOPT_ANNOTATION_DB));
 	config_seenstate_db =
 	    cyrusdb_fromname(config_getstring(IMAPOPT_SEENSTATE_DB));
+	config_mboxkey_db =
+	    cyrusdb_fromname(config_getstring(IMAPOPT_MBOXKEY_DB));
 	config_duplicate_db =
 	    cyrusdb_fromname(config_getstring(IMAPOPT_DUPLICATE_DB));
 	config_tlscache_db =
 	    cyrusdb_fromname(config_getstring(IMAPOPT_TLSCACHE_DB));
-#ifdef WITH_PTS
 	config_ptscache_db =
 	    cyrusdb_fromname(config_getstring(IMAPOPT_PTSCACHE_DB));
-#endif
 
 	/* configure libcyrus as needed */
 	libcyrus_config_setstring(CYRUSOPT_CONFIG_DIR, config_dir);
@@ -206,10 +208,14 @@ int cyrus_init(const char *alt_config, const char *ident, unsigned flags)
 				  config_getenum(IMAPOPT_VIRTDOMAINS));
 	libcyrus_config_setint(CYRUSOPT_BERKELEY_CACHESIZE,
 			       config_getint(IMAPOPT_BERKELEY_CACHESIZE));
+	libcyrus_config_setstring(CYRUSOPT_AUTH_MECH,
+				  config_getstring(IMAPOPT_AUTH_MECH));
 	libcyrus_config_setint(CYRUSOPT_BERKELEY_LOCKS_MAX,
 			       config_getint(IMAPOPT_BERKELEY_LOCKS_MAX));
 	libcyrus_config_setint(CYRUSOPT_BERKELEY_TXNS_MAX,
 			       config_getint(IMAPOPT_BERKELEY_TXNS_MAX));
+	libcyrus_config_setstring(CYRUSOPT_DELETERIGHT,
+				  config_getstring(IMAPOPT_DELETERIGHT));
 
 	/* Not until all configuration parameters are set! */
 	libcyrus_init();
@@ -244,7 +250,7 @@ void global_sasl_init(int client, int server, const sasl_callback_t *callbacks)
     }
 
     if(server && sasl_server_init(callbacks, "Cyrus")) {
-	fatal("could not init sasl (client)", EC_SOFTWARE);
+	fatal("could not init sasl (server)", EC_SOFTWARE);
     }
 }
 
@@ -488,7 +494,7 @@ static int acl_ok(const char *user, struct auth_state *authstate)
 					     bufuser, inboxname);
 
     if (r || !authstate ||
-	mboxlist_lookup(inboxname, NULL, &acl, NULL)) {
+	mboxlist_lookup(inboxname, &acl, NULL)) {
 	r = 0;  /* Failed so assume no proxy access */
     }
     else {
