@@ -37,7 +37,7 @@
 # AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
 # OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #
-# $Id: Shell.pm,v 1.31.2.5 2006/02/07 18:57:21 murch Exp $
+# $Id: Shell.pm,v 1.31.2.7 2006/03/31 19:22:35 murch Exp $
 #
 # A shell framework for Cyrus::IMAP::Admin
 #
@@ -126,7 +126,7 @@ my %builtins = (exit =>
 		  [\&_sc_info, '[mailbox]',
 		   'display mailbox/server metadata'],
 		mboxcfg =>
-		  [\&_sc_mboxcfg, 'mailbox [comment|news2mail|expire|sieve|squat] value',
+		  [\&_sc_mboxcfg, 'mailbox [comment|condstore|news2mail|expire|sieve|squat] value',
 		   'configure mailbox'],
 		mboxconfig => 'mboxcfg',
 		reconstruct =>
@@ -159,6 +159,14 @@ my %builtins = (exit =>
 		   '[--partition partition] mailbox server [partition]',
 		   'transfer (relocate) a mailbox to a different server'],
 		xfer => 'xfermailbox',
+		subscribe =>
+                  [\&_sc_subscribe, '[mailbox]',
+                    'subscribe to a mailbox'],
+                sub => 'subscribe',
+		unsubscribe =>
+		  [\&_sc_unsubscribe, '[mailbox]',
+		     'unsubscribe from a mailbox'],
+		unsub => 'unsubscribe',
 		#? alias
 		#? unalias
 		#? load
@@ -1355,6 +1363,60 @@ sub _sc_info {
   0;
 }
 
+sub _sc_subscribe {
+  my ($cyrref, $name, $fh, $lfh, @argv) = @_;
+  my (@nargv, $opt);
+  shift(@argv);
+  while (defined ($opt = shift(@argv))) {
+    # gack.  bloody tcl.
+    last if $opt eq '--';
+    if ($opt =~ /^-/) {
+      die "usage: subscribe [mailbox]\n";
+    }
+    else {
+      push(@nargv, $opt);
+      last;
+    }
+  }
+  push(@nargv, @argv);
+  if (!$cyrref || !$$cyrref) {
+    die "subscribe: no connection to server\n";
+  }
+  $$cyrref->subscribe(@nargv);
+  if (defined $$cyrref->error) {
+    $lfh->[2]->print($$cyrref->error, "\n");
+    return 1;
+  }
+  0;
+}
+
+sub _sc_unsubscribe {
+  my ($cyrref, $name, $fh, $lfh, @argv) = @_;
+  my (@nargv, $opt);
+  shift(@argv);
+  while (defined ($opt = shift(@argv))) {
+    # gack.  bloody tcl.
+    last if $opt eq '--';
+    if ($opt =~ /^-/) {
+      die "usage: unsubscribe [mailbox]\n";
+    }
+    else {
+      push(@nargv, $opt);
+      last;
+    }
+  }
+  push(@nargv, @argv);
+  if (!$cyrref || !$$cyrref) {
+    die "unsubscribe: no connection to server\n";
+  }
+  $$cyrref->unsubscribe(@nargv);
+  if (defined $$cyrref->error) {
+    $lfh->[2]->print($$cyrref->error, "\n");
+    return 1;
+  }
+  0;
+}
+
 sub _sc_mboxcfg {
   my ($cyrref, $name, $fh, $lfh, @argv) = @_;
   my (@nargv, $opt);
@@ -1362,7 +1424,7 @@ sub _sc_mboxcfg {
   while (defined ($opt = shift(@argv))) {
     last if $opt eq '--';
     if ($opt =~ /^-/) {
-      die "usage: mboxconfig mailbox [comment|news2mail|expire|sieve|squat] value\n";
+      die "usage: mboxconfig mailbox [comment|condstore|news2mail|expire|sieve|squat] value\n";
     }
     else {
       push(@nargv, $opt);
@@ -1371,7 +1433,7 @@ sub _sc_mboxcfg {
   }
   push(@nargv, @argv);
   if (@nargv < 2) {
-    die "usage: mboxconfig mailbox [comment|news2mail|expire|sieve|squat] value\n";
+    die "usage: mboxconfig mailbox [comment|condstore|news2mail|expire|sieve|squat] value\n";
   }
   if (!$cyrref || !$$cyrref) {
     die "mboxconfig: no connection to server\n";
@@ -1601,6 +1663,10 @@ The currently supported attributes are:
 
 Sets a comment or description associated with the mailbox.
 
+=item C<condstore>
+
+Enables the IMAP CONDSTORE extension (modification sequences) on the mailbox.
+
 =item C<expire>
 
 Sets the number of days after which messages will be expired from the mailbox.
